#ifndef INC_I2C_FILTER_H_
#define INC_I2C_FILTER_H_

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "fft.h"
#include "ift.h"
#include <string.h>
#include "arm_math.h"
#include "arm_const_structs.h"
#include "arm_common_tables.h"

#define ARM_MATH_CM7 1
#define __FSU_PRESENT 1
#define pi 3.14159265358979323846
#define fft_size 512
#define n_freqs ((fft_size/2)+1)
#define hop (fft_size/2)
#define audio_len (6*fft_size)
#define n_frames ((int)(((audio_len - fft_size) / hop)) + 1)
#define num_bands 8
#define buffer_add (audio_len-fft_size)
#define buffer_slice (audio_len-hop)
#define FS 44000
#define NUM_SECONDS 5
#define VOLUME 50

void normalize(float* audio_sig);

void create_filter(float* B, float* A, float complex* filt);

float fn_sinh(float x);

void true_bell_filter(float K, float fc, float BW, float fs, float complex* freq_filter);

void low_shelf(float K, float fc, float S, float fs, float complex* freq_filter);

void high_shelf(float K, float fc, float S, float fs, float complex* freq_filter);

void construct_filterbank(float* gains, float* center_frequencies, float* band_widths,
		float fs, float complex filterbank[num_bands][n_freqs], float complex big_filter[n_freqs]);

void hanning_window(float win_len, float* win_a);

void stft(float* audio_signal, float complex X[n_frames][n_freqs]);

void istft(float complex X[n_frames][n_freqs], float* audio_signal);

void filter_input(float* audio_signal, float* final_signal,
		float complex big_filter[n_freqs]);

int initialize_with_chirp(float* sig, int start, int len, int c);

int save_to_file_continuous(float* final_signal, int aud_len);

void construct_fast_filterbank(float* gains, float* center_frequencies, float* band_widths, float fs,
		arm_fir_instance_f32* filterbank[num_bands], float states[num_bands][fft_size*2-1]);

void high_shelf_fast(float K, float fc, float S, float fs, arm_fir_instance_f32* freq_filter, float pState[fft_size*2-1]);

void low_shelf_fast(float K, float fc, float S, float fs, arm_fir_instance_f32* freq_filter, float pState[fft_size*2-1]);

void true_bell_filter_fast(float K, float fc, float BW, float fs, arm_fir_instance_f32* freq_filter, float pState[fft_size*2-1]);

void create_filter_fast(float* B, float* A, arm_fir_instance_f32* filt, float pState[fft_size*2-1]);

void fast_filter(float* audio_signal, float* final_signal, float complex filterbank[num_bands][n_freqs]);

// int save_to_file(float* final_signal, int audio_len, int b_dx);

// int save_to_file_filterbank(float complex** filterbank, int num_bands, float num_freqs);

#endif /* INC_I2C_FILTER_H_ */
