#include "i2c-codec.h"

#define SLAVE_ADDRESS_CODEC 0x98 // change this according to ur setup

HAL_StatusTypeDef write_i2c_register(I2C_HandleTypeDef *hi2c1, uint8_t register_pointer, uint8_t register_value)
{
    uint8_t data[2];

    data[0] = register_pointer;     // register address
    data[1] = register_value;       // register value

    HAL_StatusTypeDef res = HAL_I2C_Master_Transmit (hi2c1, SLAVE_ADDRESS_CODEC, data, 2, 100);

    return res;
}

HAL_StatusTypeDef read_i2c_register(I2C_HandleTypeDef *hi2c1, uint8_t register_pointer, uint8_t* receive_buffer)
{
    // first set the register pointer to the register wanted to be read
	HAL_StatusTypeDef res = HAL_I2C_Master_Transmit(hi2c1, SLAVE_ADDRESS_CODEC, &register_pointer, 1, 100);  // note the & operator which gives us the address of the register_pointer variable
	if (res != HAL_OK)
		return res;
    // receive the 1 x 8bit data into the receive buffer
    res = HAL_I2C_Master_Receive(hi2c1, SLAVE_ADDRESS_CODEC, receive_buffer, 1, 100);
    return res;
}

HAL_StatusTypeDef codec_init(I2C_HandleTypeDef *hi2c1)
{
    uint8_t register_pointer;
    uint8_t register_value;

    register_pointer = 0x03;
    register_value = 0x18;
    HAL_StatusTypeDef res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x04;
    register_value = 0x10;
    res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x05;
    register_value = 0x00;
    res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x06;
    register_value = 0x20;
    res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x09;
    register_value = 0x19;
    res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x0E;
    register_value = 0x0F;
    res = write_i2c_register(hi2c1, register_pointer, register_value);
    if (res != HAL_OK)
    		return res;

    // Clear PDN bit, kickstarting the codec
	HAL_Delay(50);
    register_pointer = 0x02;
    register_value = 0x00;
    // res = write_i2c_register(hi2c1, register_pointer, register_value);
    return res;
}

HAL_StatusTypeDef codec_test(I2C_HandleTypeDef *hi2c1)
{
	// read each register we've written to during codec_init()
	// debug and check register buffer after every read and ensure
	// they match the init values
    uint8_t register_pointer;
    uint8_t register_buffer[1];

    register_pointer = 0x03;
    HAL_StatusTypeDef res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x04;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x05;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x06;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x09;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x0E;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    if (res != HAL_OK)
    		return res;

	HAL_Delay(50);
    register_pointer = 0x02;
    res = read_i2c_register(hi2c1, register_pointer, register_buffer);
    return res;
}
