/*
 * i2s.c
 *
 *  Created on: Apr 10, 2021
 *      Author: admin
 */

#include "i2s_transfer.h"

HAL_StatusTypeDef receive_I2S(I2S_HandleTypeDef* hi2s2, float* right, float* left, uint16_t len)
{
	while ((hi2s2)->State != HAL_I2S_STATE_READY)
		;

	// length is x4 because we have two channels, and we need two 16-bit chunks
	// to write a 32-bit chunk to our left and right buffers
	uint16_t temp_buffer[len*4];

	HAL_StatusTypeDef result = HAL_I2S_Receive(hi2s2, temp_buffer, len*4, 100);

	for (int i = 0; i < len; i++) {
		int lefto_u = ((int)(temp_buffer[4*i])) << 16;
		int lefto_l = (int)(temp_buffer[4*i+1]);
		int righto_u = ((int)(temp_buffer[4*i+2])) << 16;
		int righto_l = (int)(temp_buffer[4*i+3]);

		left[i] = (float)((lefto_u & 0xFFFF0000) | (lefto_l & 0x0000FFFF));
		right[i] = (float)((righto_u & 0xFFFF0000) | (righto_l & 0x0000FFFF));
		// keep top 24 bits because data received is only 24-bit
		// left[i] = (float)((((int)(temp_buffer[4*i]) << 16) & 0xFFFF0000) | (((int)temp_buffer[4*i+1]) & 0x0000FFFF));
		// right[i] = (float)((((int)(temp_buffer[4*i+2]) << 16) & 0xFFFF0000) | (((int)temp_buffer[4*i+3]) & 0x0000FFFF));
		// left[i] = (float)(((int)(temp_buffer[4*i]) << 8) | ((int)(temp_buffer[4*i+1] & 0xFF00) >> 8));
		// right[i] = (float)(((int)(temp_buffer[4*i+2]) << 8) | ((int)(temp_buffer[4*i+3] & 0xFF00) >> 8));
	}
	return result;
}

HAL_StatusTypeDef transmit_I2S(I2S_HandleTypeDef* hi2s2, float* right, float* left, uint16_t len)
{
	while ((hi2s2)->State != HAL_I2S_STATE_READY)
			;

	uint16_t temp_buffer[4];
	HAL_StatusTypeDef result;

	for (int i = 0; i < len; ++i) {

		int lefto = (int)(left[i]);
		int righto = (int)(right[i]);
		temp_buffer[0] = (uint16_t)(lefto >> 16);
		temp_buffer[1] = (uint16_t)(lefto);
		temp_buffer[2] = (uint16_t)(righto >> 16);
		temp_buffer[3] = (uint16_t)(righto);
		result = HAL_I2S_Transmit(hi2s2, temp_buffer, 4, 100);
		if (result != HAL_OK)
			return result;

		/*
		temp_buffer[0] = (uint16_t)((((int)left[i]) & 0xFFFF0000) >> 16);
		temp_buffer[1] = (uint16_t)((((int)left[i]) & 0x0000FFFF));
		temp_buffer[2] = (uint16_t)((((int)right[i]) & 0xFFFF0000) >> 16);
		temp_buffer[3] = (uint16_t)((((int)right[i]) & 0x0000FFFF));

		// keep the top-most 16 bits (24:8) as the MSB
		temp_buffer[i] = (uint16_t)((((int)left[i]) & 0x00FFFF00) >> 8);
		// keep the bottom-most 8 bits (8:1) as the LSB
		// this discards the top 8 bits
		temp_buffer[i+1] = (uint16_t)((((int)left[i]) & 0x000000FF) << 8);

		temp_buffer[i+2] = (uint16_t)((((int)right[i]) & 0x00FFFF00) >> 8);
		temp_buffer[i+3] = (uint16_t)((((int)right[i]) & 0x000000FF) << 8);
		*/
	}
	return result;

}

